/*
 * BTLMainWindowController.h
 * StoryWriter
 *
 * Controller for the main (editor) window of StoryWriter
 * 
 * Created by Matthew Christensen on Tue Aug 23 2004
 * Copyright © 2004 Matthew Christensen. All rights reserved.
 */

#import <Cocoa/Cocoa.h>
#import "BTLTextView.h"

@class BTLStyle;
@class BTLTextView;
@class BTLPreviewView;
@class BTLLayoutManager;
@class BTLCard;

// Menu tag constants
#define DOCUMENT_INFO_ITEM_TAG	1001
#define ADD_CARD_ITEM_TAG	1002
#define EDIT_CARD_ITEM_TAG	1003
#define DELETE_CARD_ITEM_TAG	1004
#define IMPORT_ITEM_TAG			1005
#define EXPORT_ITEM_TAG			1006
#define STYLE_ITEM_TAG			1007
#define PRINT_ITEM_TAG			1009
#define STYLE_CHANGE_TAG		1010
#define STYLE_CONVERT_TAG		1012
#define INSERT_PAGE_BREAK_TAG	1013

#define TEXT_EXPORT_TAG			1
#define RTF_EXPORT_TAG			2

#define DragDropBTLCardPboardType @"DragDropBTLCardPboardType"

@interface BTLMainWindowController : NSWindowController <BTLTextViewDelegate>
{
	NSTextStorage *documentText;
	
	// For displaying progress
	IBOutlet NSWindow *progressWindow;
	IBOutlet NSProgressIndicator *progressIndicator;
	IBOutlet NSTextField *progressLabel;
	SEL progressCancelSelector;
	id progressCancelObject;
	
	// Preview area
	IBOutlet NSScrollView *previewScrollView;
	IBOutlet BTLPreviewView *previewView;
	BTLLayoutManager *previewLayoutManager;
	
	// Scene drawer
	IBOutlet NSDrawer *cardDrawer;
	IBOutlet NSTableView *cardTableView;
	IBOutlet NSPopUpButton *cardPopUpButton;

	// Used while typing to track our status
	// should this be part of the model?
	BTLStyle *lastStyle;
	
	// For toolbar
	NSMutableDictionary *toolbarItems;
	IBOutlet NSView *stylePopUpView;
	NSPopUpButton *stylePopUp;
	bool ignoreStyleChange;
	
	// Used to track the state of our layout
	// progress sheet
	bool layoutProgressSheetOpen;
	
	// Current selection range
	NSRange selectedRange;
	
	// Use to disable auto-scrolling on card selection
	bool ignoreNextCardSelection;
	
	// We use this to have a BTLTextView. In the current design there are multiple
	// BTLTextViews, but we only need one
	BTLTextView *textEditor;
	
	// Any dragged objects
	NSArray *draggedItems;

	// Edit card sheet
	IBOutlet NSPanel *editCardSheet;
	IBOutlet NSTextField *cardNameField;
	IBOutlet NSTextView *cardNotesView;
	IBOutlet NSButton *cardDisplayNotesCheckbox;
	
	// View for export panel
	IBOutlet NSView *exportFormatView;
	IBOutlet NSPopUpButton *exportFormatPopUp;
	
	// For demo - did we hit our page limit?
	bool hitDemoLimit;
	
	// Set to true after window is fully initialized
	bool windowWasSetup;
	
	unsigned pageCount;
}

#pragma mark Interface management

// Manage the enabling and disabling of menus
- (BOOL) validateMenuItem: (NSMenuItem *) menuItem;

#pragma mark Utilities
// When the data model changes, this is called to set up all
// parts that rely on it
- (void) setupFromDataModel: (NSNotification *) note;

// Shortcut to our model's text object
- (BTLTextStorage *) text;

// Force the layout of pages
- (void) forceLayout;

- (void) setupPreview;

// Close our progres sheet by setting the flag to false
- (void) closeProgressSheet;

// Make sure all pages are rendered in the preview
- (void) renderAllPreviewPages;

- (void) changeStyle: (BTLStyle *) style;

// Remove any pages that are unused
- (void) purgeUnusedPages;

// Given a length, add the appropriate number of pages
// that this text will probably require.
- (void) addEstimatedPageForLength: (unsigned) len;

// Select the location, scroll to it, and layout the text
// at the location
- (void) selectScrollToAndLayout: (unsigned) loc;

#pragma mark Printing

// Called after the print runs. We restore any non-printing features
- (void)printOperationDidRun:(NSPrintOperation *)printOperation success:(BOOL)success contextInfo:(void *)contextInfo;

// enter printing mode, which involves forcing layout, stripping non-printing
// text, switching to default font and
- (void) enterPrintingMode;

// Restore anything hidden/removed by enterPrintingMode
- (void) exitPrintingMode;


#pragma mark Actions


// Cancel the action our progress sheet is currently handling
- (IBAction) progressSheetCancelAction: (id) sender;

- (IBAction) printDocument:(id)sender;

- (IBAction) showDocumentInfo: (id) sender;

// Inserts a page break at the selection
- (IBAction) insertPageBreak: (id) sender;

#pragma mark BTLTextView delegate

- (void) startProgressSheet: (int) max  label: (NSString *) label cancelSelector: (SEL) sel cancelObject: (id) obj;
- (void) updateProgressSheet: (int) value;
- (void) endProgressSheet;

#pragma mark NSTextView delegate

// Used to intercept text typed or pasted into the textview, so we can pass it off to the template
// for formatting.
- (BOOL)textView:(NSTextView *)aTextView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;

// Whenever the selection changes, update the style menu
- (void)textViewDidChangeSelection:(NSNotification *)aNotification;

#pragma mark NSLayoutManager delegate
- (void)layoutManager:(NSLayoutManager *)aLayoutManager didCompleteLayoutForTextContainer:(NSTextContainer *)aTextContainer atEnd:(BOOL)flag;

#pragma mark Import/Export
- (void) importText: (NSString *) text;

#pragma mark Utilities
- (void) addPageFromNotification: (NSNotification *) note;
- (void) addEditorPages: (unsigned) count;


@end

@interface BTLMainWindowController (BTLToolbarController)

#pragma mark Style change

- (IBAction) switchToAction: (id) sender;
- (IBAction) switchToParenthetical: (id) sender;
- (IBAction) switchToDialog: (id) sender;
- (IBAction) switchToCharacter: (id) sender;
- (IBAction) switchToTransition: (id) sender;
- (IBAction) insertINT: (id) sender;
- (IBAction) insertEXT: (id) sender;
- (IBAction) insertComment: (id) sender;

#pragma mark Initialization

// Setup our toolbar items in a dictionary
// use a #define statement to pull things off if we are in demo mode
- (void) setupToolbar;

#pragma mark Validation

// Check to see if a toolbar item is enabled or disabled
-(BOOL)validateToolbarItem:(NSToolbarItem*)toolbarItem;

@end


@interface BTLMainWindowController (BTLCardDrawerController)

#pragma mark Actions

- (IBAction) openCardDrawer: (id) sender;
- (IBAction) closeCardDrawer: (id) sender;
- (IBAction) toggleCardDrawer: (id) sender;

- (IBAction) deleteCard: (id) sender;
- (IBAction) editCard: (id) sender;

- (IBAction) filterCards: (id) sender;


#pragma mark Utilities

// Called when card notification recieved
- (void) selectCard: (NSNotification *) notification;

// Convienence method to return teh cards array from the model
- (NSMutableArray *) cards;

// Currently selected card. Nil if no card selected.
- (BTLCard *) selectedCard;

// Cause the card table view to be redrawn
- (void) redrawCardTableView: (NSNotification *) notification;

// Ignore the next selection change on the card table view
- (void) ignoreNextCardSelection;

// Actually handle removing the account channel
- (void) handleCardDelete: (NSWindow *)sheet
				  returnCode: (int)returnCode
				 contextInfo: (void *)contextInfo;

#pragma mark NSTableDataSource implementation

// Returns number of cards
- (int)numberOfRowsInTableView:(NSTableView *)aTableView;

// Return the value for a given table cell
- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;

// Set the value for a given cell (on edit)
- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;


// Accept a drop onto this table
- (BOOL)tableView:(NSTableView *) aTableView acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation;

// Called by drag. Start drag-and-drop action
- (BOOL)tableView:(NSTableView *) aTableView writeRows:(NSArray *)rows toPasteboard:(NSPasteboard *)pboard;

// Check that drag is ok
- (NSDragOperation)tableView:(NSTableView *) aTableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation;

#pragma mark NSTableView delegate

// Should we be able to edit a given cell?
- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;

// Something in a table view was selected. We use to scroll to the appropriate scene
- (void)tableViewSelectionDidChange:(NSNotification *)aNotification;

@end

@interface BTLMainWindowController (BTLEditScenePanelController)

#pragma mark Actions

- (void) showEditCardSheet;
- (IBAction) editCardSheetOKAction: (id) sender;
- (IBAction) editCardSheetCancelAction: (id) sender;

@end
