/*
 * BTLStyle.h
 * StoryWriter
 *
 * Describes a particular style associated with a template. 
 * Beyond having a name, the style has several pieces of formatting information.
 *
 * Created by Matthew Christensen on Tue Aug 24 2004
 * Copyright © 2004 Matthew Christensen. All rights reserved.
 */

#import <Cocoa/Cocoa.h>
#import "BTLDocumentInfo.h"

// Attribute that sets the style name
#define BTLSTYLE_ATTRIBUTE @"BTLStyle"

#define NOBREAK_ATTRIBUTE @"BTLNoBreak"
#define BREAK_ATTRIBUTE @"BTLNoBreak"

#define INCH_IN_POINTS 72.0

@class BTLStyle;

// Structure returned by nextStyleForKeystroke
typedef struct
{
	BTLStyle *style;					// New style
	NSString *textToInsert;				// Text to insert with old style
	NSString *textToInsertAfterChange;	// Text to insert in new style
	bool paragraphStyleChange;			// True if style should be applied retroactively
	bool clearCurrentText;				// indicate to clear current text after change
	bool forceNewParagraph;				// Make sure this is separated from the previous line by a blank line 
} NextStyleStruct;


@interface BTLStyle : NSObject
{
	// Used to group styles
	int family;
	
	// Distance, in points, from left (relative)
	float leftMargin;

	// Distance, in points, from right (relative)
	float rightMargin;
	
	// True if style should display as all caps.
	bool allCaps;
	
	// True if firstchar in sentence should be capitlized
	bool capitalize;
	
	// Font to use. If nil, use default.
	NSFont *font;
	
	// If true, user can see style in the style menu. 
	// Otherwise it’s a “hidden” style, only switched to by the code itself
	bool visibleToUser;
	
	// If true, words typed in this style will be added to the autocomplete list
	// default is false
	bool addToAutocomplete;
	
	// Name of this style
	NSString * name;
		
	// Name this shows up as to user. If the style is hidden, you want this to be a style 
	//that actually appears in the style menu.
	NSString *displayName;
	
	// Used to set attributes
	NSMutableDictionary *attributes;
	
	// If non-nil, the subsequent paragraph should be changed to this style
	BTLStyle *forceNextStyleTo;
	
	// Are we a style that needs to be its own paragraph?
	bool paragraphStyle;
	
	// Set the text alignment for this style
	int textAlignment;
	
	// Are we linked to card ?
	bool cardLinked;
	
	// Styles to change to from this one
	BTLStyle *tabStyle;
	BTLStyle *paragraphChangeStyle;
	
	// For checking for paragraph changes
	NSMutableCharacterSet *paragraphChangeCharacters;
	
	bool noBreak;
}

#pragma mark Initialzation
// Designated initializer
- init;

#pragma mark Accessors

// Return the style name
- (NSString *) name;

// Return the style's display name
- (NSString *) displayName;

// Return the style's family
- (int) family;

// Return true if in the same family as the other style. No style never matches
- (bool) sameFamilyAs: (BTLStyle *) style;

// Return true if user can pick this style
- (bool) visibleToUser;

// Return true if words in this style should be added to autocomplete
- (bool) addToAutocomplete;

// If non-nil, the subsequent paragraph should be changed to this style
- (BTLStyle *) forceNextStyleTo;

// Style to change to on tab
- (BTLStyle *) tabStyle;
- (void) setTabStyle: (BTLStyle *) s;

// Style to change to on a paragraph change
- (BTLStyle *) paragraphChangeStyle;
- (void) setParagraphChangeStyle: (BTLStyle *) s;

// Should the generic autocomplete items be used?
- (bool) useGeneralAutocomplete;

// Autocomplete items to be used
- (NSArray *) autocompleteItems;

// Are lines of this style linked to cards?
- (bool) cardLinked;

#pragma mark Formatting

// Handle any post-style change processing, like adding autocompletes
- (void) postStyleChangeProcess: (NSString *) currentText
				   documentInfo: (BTLDocumentInfo *) doc;

// Given a string, return a dictionary with the appropriate formatting attributes for this style.
- (NSDictionary *) attributesForStyle: (BTLDocumentInfo *) info;

// Allow the style to determine a style change
- (NextStyleStruct) checkForStyleChange: (unichar) key 
						   currentText: (NSString *) currentText
						   documentInfo: (BTLDocumentInfo *) doc;

#pragma mark Utilities

// Return a capitalized string, based on the prefix string
- (NSString *) capitalize: (NSString *) str prefix: (NSString *) prefix;

@end
