/*
 * BTLTextView.h
 * StoryWriter
 *
 * Subclass of BTLTextView that properly handles Storywriter styles.
 * 
 * Created by Matthew Christensen on Friday Aug 27 2004
 * Copyright © 2004 Matthew Christensen. All rights reserved.
 */

#import <Cocoa/Cocoa.h>
#import "BTLTextViewDelegate.h"

@class BTLTemplate;
@class BTLStyle;
@class BTLDocumentInfo;
@class BTLTextStorage;
@class BTLDataModel;

@interface BTLTextView : NSTextView 
{
	BTLTemplate *template;
	BTLDocumentInfo *documentInfo;
	
	BTLStyle *currentStyle;
	NSMutableString *currentText;
	
	// List of current words offered as completions
	NSMutableArray *completionWords;

	unsigned int nextCharLocation;
	
	bool progressOpen;
	
	// Stores the selected font for display, so
	// we can restore it after a reset to the template font
	NSFont *displayFont;

	// Temporary disable autocomplete
	bool temporaryDisableAutocomplete;
	
	// Used for completions
	NSRange completionRange;
	NSMutableCharacterSet *wordBoundaryCharacters;
	NSString *originalString;
}


#pragma mark Initialization

// Create a test version of the view
+ (BTLTextView *) createTestView: (BTLDocumentInfo *) info  template: (BTLTemplate *) t;

// Designated initializer
- initWithFrame: (NSRect) frame textContainer: (NSTextContainer *) container;

// Sets up this tv with the template, sections, and info from a data model
- (void) updateFromDataModel: (BTLDataModel *) m;

#pragma mark Accessors

- (void) setTemplate: (BTLTemplate *) t;
- (BTLTemplate *) template;

- (void) setDocumentInfo: (BTLDocumentInfo *) t;
- (BTLDocumentInfo *) documentInfo;

- (void) setCurrentStyle: (BTLStyle *) s;
- (BTLStyle *) currentStyle;

- (void) cancelPaste;

#pragma mark Text management

// Paginate (if active and necessary) from a given point
- (void) paginateFromLocation: (unsigned) paginateFrom;

// Import text from a string to a screenplay.
// if replaceExisting is true, all text in storage will be replaced.
// Otherwise, just selection
- (void) importText: (NSString *) str replaceExisting: (bool) replaceExisting;

// Insert some text into the TextView at a given location. This will then
// use the styles to format it
- (void) insertText: (NSString *) text atLocation: (unsigned) location;

// Override paste for formatting purposes
- (void)paste: (id) sender;

// Return the style for a given index
- (BTLStyle *) styleForLocation: (unsigned) location;

// Set the style for a given range
- (void) setStyle: (BTLStyle *) newstyle forRange: (NSRange) range;

// Set our font for all text
- (void) setEditorFont: (NSFont *) f;

// Set everything to the template font
- (void) useTemplateFont;

// Set everything to the display font
- (void) useDisplayFont;

// Change the text in a range to a given style
- (BTLStyle *) changeRange: (NSRange) r 
				   toStyle: (BTLStyle *) style 
				  delegate: (id <BTLTextViewDelegate>) obj
					   end: (unsigned *) end;

// Used to track deletes for autocap
- (void)deleteBackward:(id)sender;

// Used to track deletes for autocap disable
// If you backspace/delete a char, it won't capitalize the next time you type it
- (void)deleteForward:(id)sender;

// Use the layout manager and layout the selected text
- (void) layoutSelectedText;

// Use the layout manager and layout the visible text
- (void) layoutVisibleText;


#pragma mark NSFontManager delgate commands

- (void)changeFont:(id)sender;


@end
